from pydantic import BaseModel, Field

from oshconnect.csapi4py.constants import GeometryTypes


# TODO: Add specific validations for each type
# TODO: determine if serializing 'shapely' objects gives valid JSON structures from our own serialization
class Geometry(BaseModel):
    """
    A class to represent the geometry of a feature
    """
    type: GeometryTypes = Field(...)
    coordinates: list
    bbox: list = None



from __future__ import annotations

from pydantic import BaseModel, Field, HttpUrl

from oshconnect.datamodels.api_utils import Link, URI
from oshconnect.datamodels.geometry import Geometry


class SystemEventOMJSON(BaseModel):
    """
    A class to represent the schema of a system event
    """
    label: str = Field(...)
    description: str = Field(None)
    definition: HttpUrl = Field(...)
    identifiers: list = Field(None)
    classifiers: list = Field(None)
    contacts: list = Field(None)
    documentation: list = Field(None)
    time: str = Field(...)
    properties: list = Field(None)
    configuration: dict = Field(None)
    links: list[Link] = Field(None)


class SystemHistoryGeoJSON(BaseModel):
    """
    A class to represent the schema of a system history
    """
    type: str = Field(...)
    id: str = Field(None)
    properties: SystemHistoryProperties = Field(...)
    geometry: Geometry = Field(None)
    bbox: list = Field(None)
    links: list[Link] = Field(None)


class SystemHistoryProperties(BaseModel):
    feature_type: str = Field(...)
    uid: URI = Field(...)
    name: str = Field(...)
    description: str = Field(None)
    asset_type: str = Field(None)
    valid_time: list = Field(None)
    parent_system_link: str = Field(None, serialization_alias='parentSystem@link')
    procedure_link: str = Field(None, serialization_alias='procedure@link')



from pydantic import BaseModel, Field, field_validator, SerializeAsAny

from oshconnect.csapi4py.constants import ObservationFormat
from oshconnect.datamodels.encoding import Encoding
from oshconnect.datamodels.swe_components import AnyComponentSchema


class DatastreamSchema(BaseModel):
    """
    A class to represent the schema of a datastream
    """
    obs_format: str = Field(..., serialization_alias='obsFormat')


class SWEDatastreamSchema(DatastreamSchema):
    encoding: SerializeAsAny[Encoding] = Field(...)
    record_schema: SerializeAsAny[AnyComponentSchema] = Field(..., serialization_alias='recordSchema')

    @field_validator('obs_format')
    @classmethod
    def check_check_obs_format(cls, v):
        if v not in [ObservationFormat.SWE_JSON.value, ObservationFormat.SWE_CSV.value,
                     ObservationFormat.SWE_TEXT.value, ObservationFormat.SWE_BINARY.value]:
            raise ValueError('obsFormat must be on of the SWE formats')
        return v



from datetime import datetime
from typing import Union

from pydantic import BaseModel, Field


class CommandJSON(BaseModel):
    """
    A class to represent a command in JSON format
    """
    control_id: str = Field(None, serialization_alias="control@id")
    issue_time: Union[str, float] = Field(datetime.now().isoformat(), serialization_alias="issueTime")
    sender: str = Field(None)
    params: Union[dict, list, int, float, str] = Field(None)



from __future__ import annotations

from typing import Union

from pydantic import BaseModel, Field, SerializeAsAny

from oshconnect.datamodels.encoding import Encoding
from oshconnect.datamodels.swe_components import AnyComponentSchema


class ControlStreamJSONSchema(BaseModel):
    """
    A class to represent the schema of a control stream
    """
    id: str = Field(None)
    name: str = Field(...)
    description: str = Field(None)
    deployment_link: str = Field(None, serialization_alias='deployment@link')
    ultimate_feature_of_interest_link: str = Field(None, serialization_alias='ultimateFeatureOfInterest@link')
    sampling_feature_link: str = Field(None, alias='samplingFeature@link')
    valid_time: list = Field(None, serialization_alias='validTime')
    input_name: str = Field(None, serialization_alias='inputName')
    links: list = Field(None)
    control_stream_schema: SerializeAsAny[Union[SWEControlChannelSchema, JSONControlChannelSchema]] = Field(...,
                                                                                                            serialization_alias='schema')


class SWEControlChannelSchema(BaseModel):
    """
    A class to represent the schema of a control channel
    """
    command_format: str = Field("application/swe+json", serialization_alias='commandFormat')
    encoding: SerializeAsAny[Encoding] = Field(...)
    record_schema: SerializeAsAny[AnyComponentSchema] = Field(..., serialization_alias='recordSchema')


class JSONControlChannelSchema(BaseModel):
    command_format: str = Field("application/cmd+json", serialization_alias='commandFormat')
    params_schema: SerializeAsAny[AnyComponentSchema] = Field(..., serialization_alias='paramsSchema')



# import api_utils
# import commands
# import control_streams
# import datastreams
# import encoding
# import geometry
# import network_properties
# import observations
# import swe_components
# import system_events_and_history



from datetime import datetime
from typing import Union, List

from pydantic import BaseModel, Field

from oshconnect.datamodels.api_utils import Link


class ObservationOMJSONInline(BaseModel):
    """
    A class to represent an observation in OM-JSON format
    """
    datastream_id: str = Field(None, serialization_alias="datastream@id")
    foi_id: str = Field(None, serialization_alias="foi@id")
    phenomenon_time: str = Field(None, serialization_alias="phenomenonTime")
    result_time: str = Field(datetime.now().isoformat(), serialization_alias="resultTime")
    parameters: dict = Field(None)
    result: Union[int, float, str, dict, list] = Field(...)
    result_links: List[Link] = Field(None, serialization_alias="result@links")
